%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0. Housekeeping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
close all;
set_dynare_seed(31415927);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

@#for states in 1:51
    var y_@{states} cm_@{states} cn_@{states} inv_@{states} k_@{states} hn_@{states} sn_@{states} sm_@{states} hm_@{states} logy_@{states} logcm_@{states} logi_@{states} logkm_@{states} logkn_@{states} loghm_@{states} loghi_@{states} loghc_@{states} loghn_@{states} loglei_@{states} logw_@{states} ;
@#endfor

@#for states in 1:51
   varexo en_@{states} em_@{states} merror1_@{states} merror2_@{states};
@#endfor

parameters a b beta delta eta rho stden stdem gamma eps
           knbar kmbar ybar ibar hmbar hnbar theta stdmfactor
  @#for states in 1:51
      stdm_@{states}
  @#endfor
;

% See Benhabib et al and Aguiar et al for these parameter 
% values 

theta=0.36;
beta=0.96;
delta=0.1;
eta=0.08;
eps=2;
rho=0.95^4;
hmbar=0.2004;
hnbar=0.1148;

% These next three starting values won't effect the 
% eventual estimated values.  Just initalize some values 

gamma=2/3;
stden=0.028;
stdem=0.028;

kmbar=hmbar*((1/beta-1+delta)/theta)^(1/(theta-1));
knbar=kmbar/hmbar*hnbar/(theta/(1-theta)*(1-eta)/eta);
ibar=delta*(knbar+kmbar);
ybar=hmbar^(1-theta)*kmbar^theta;
cmbar=ybar-ibar;
cnbar=hnbar^(1-eta)*knbar^eta;
a=1/(1+(1-theta)*(kmbar/hmbar)^theta/((1-eta)*(knbar/hnbar)^eta*(cnbar/cmbar)^(eps-1)));
b=1/(1+(1-hmbar-hnbar)*(1-a)*(1-eta)*(knbar/hnbar)^eta*cnbar^(eps-1)/(a*cmbar^eps+(1-a)*cnbar^eps));

% Standard deviation of sampling error.  
% The sampling error for the % largest state (California) was set 
% equal to 0.  The sampling error for all other states is given 
% as the predicted values of from a regression of 
% standard deviation of home work growth rates (on the left hand) 
% side versus log sample size (on the right hand side), 
% relative to the predicted value of that same regression
% for California.  

stdm_1 = .103;
stdm_2 = .218;
stdm_3 = .099;
stdm_4 = .131;
stdm_5 = .0000001;
stdm_6 = .092;
stdm_7 = .122;
stdm_8 = .202;
stdm_9 = .213;
stdm_10 = .039;
stdm_11 = .073;
stdm_12 = .201;
stdm_13 = .163;
stdm_14 = .049;
stdm_15 = .086;
stdm_16 = .118;
stdm_17 = .122;
stdm_18 = .104;
stdm_19 = .115;
stdm_20 = .171;
stdm_21 = .094;
stdm_22 = .087;
stdm_23 = .059;
stdm_24 = .082;
stdm_25 = .132;
stdm_26 = .086;
stdm_27 = .189;
stdm_28 = .148;
stdm_29 = .142;
stdm_30 = .17;
stdm_31 = .073;
stdm_32 = .148;
stdm_33 = .034;
stdm_34 = .068;
stdm_35 = .202;
stdm_36 = .053;
stdm_37 = .115;
stdm_38 = .109;
stdm_39 = .048;
stdm_40 = .187;
stdm_41 = .107;
stdm_42 = .196;
stdm_43 = .096;
stdm_44 = .02;
stdm_45 = .129;
stdm_46 = .216;
stdm_47 = .071;
stdm_48 = .083;
stdm_49 = .158;
stdm_50 = .083;
stdm_51 = .224;

% Do we allow for hours to be subject to sampling "measurement" 
% error.  Equal to 1 if yes, 0 if not.  
stdmfactor = 1; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
model(linear);

@#for states in 1:51
 log(sm_@{states})=rho*log(sm_@{states}(-1))+em_@{states}*stdem;
 log(sn_@{states})=rho*log(sn_@{states}(-1))+stden * ( gamma* em_@{states}+ sqrt(1-gamma^2)*en_@{states});
 cm_@{states}+inv_@{states}=y_@{states};
 k_@{states}=(1-delta)*k_@{states}(-1)+inv_@{states};
 y_@{states}=sm_@{states}*((k_@{states}(-1)/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) ))^theta*hm_@{states}^(1-theta);
 cn_@{states}=sn_@{states}*((k_@{states}(-1)*(1-1/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) )))^eta*hn_@{states}^(1-eta);
 (1-b)/(1-hm_@{states}-hn_@{states})=a*b*sm_@{states}*(1-theta)*((k_@{states}(-1)/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) )/hm_@{states})^theta*cm_@{states}^(eps-1)/(a*cm_@{states}^(eps)+(1-a)*cn_@{states}^(eps));
 (1-b)/(1-hm_@{states}-hn_@{states})=(1-a)*b*sn_@{states}*(1-eta)*((k_@{states}(-1)*(1-1/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) ))/hn_@{states})^eta*cn_@{states}^(eps-1)/(a*cm_@{states}^(eps)+(1-a)*cn_@{states}^(eps));
  cm_@{states}(+1)^(eps-1)*beta*(sm_@{states}(+1)*theta*((k_@{states}/(1+hn_@{states}(+1)/hm_@{states}(+1)*eta/(1-eta)*(1-theta)/theta) )/hm_@{states}(+1))^(theta-1)+1-delta)/(a*cm_@{states}(+1)^eps+(1-a)*cn_@{states}(+1)^eps)=(cm_@{states})^(eps-1)/((a*(cm_@{states})^eps+(1-a)*(cn_@{states})^eps));
 logy_@{states}=log(y_@{states});
 logcm_@{states}=log(cm_@{states});
 logi_@{states}=log(inv_@{states});
 logw_@{states}=log(y_@{states}/hm_@{states});
 logkm_@{states}=log((k_@{states}(-1)/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) ));
 logkn_@{states}=log(k_@{states}(-1)-(k_@{states}(-1)/(1+hn_@{states}/hm_@{states}*eta/(1-eta)*(1-theta)/theta) ));
 loghm_@{states}=log(hm_@{states})-log(hmbar)+stdm_@{states} *merror2_@{states}*stdmfactor  ;
 loghi_@{states}=log(hm_@{states}*inv_@{states}/y_@{states});
 loghc_@{states}=log(hm_@{states}*cm_@{states}/y_@{states});
 loghn_@{states}=log(hn_@{states})-log(hnbar) + stdm_@{states} *merror1_@{states}*stdmfactor   ;
 loglei_@{states}=log(1-hn_@{states}-hm_@{states});
@#endfor
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Computation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

initval;
@#for states in 1:51
 hn_@{states}=hnbar;
 hm_@{states}=hmbar;
 k_@{states}=knbar+kmbar;
 y_@{states}=ybar;
 cn_@{states}=cnbar; 
 cm_@{states}=cmbar; 
 inv_@{states}=ibar; 
 sn_@{states}=1;
 sm_@{states}=1;
 logy_@{states}=log(ybar);
 logcm_@{states}=log(cmbar);
 logi_@{states}=log(ibar);
 logw_@{states}=log(ybar/hmbar);
 logkm_@{states}=log(kmbar);
 logkn_@{states}=log(knbar);
 loghm_@{states}=0;
 loghi_@{states}=log(hmbar*ibar/ybar);
 loghc_@{states}=log(hmbar*cmbar/ybar);
 loghn_@{states}=0;
 loglei_@{states}=log(1-hmbar-hnbar);
@#endfor
end;

shocks;
@#for states in 1:51
  var en_@{states}; stderr 1; 
  var em_@{states}; stderr 1; 
  var merror1_@{states}; stderr 1;
  var merror2_@{states}; stderr 1;
@#endfor
end;

varobs  
 @#for states in 1:51
      loghm_@{states}, loghn_@{states}
 @#endfor
;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 4. Estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

estimated_params;
//  PARAM NAME, INITVAL, LB, UB
     stden , 0.15 , 0.00005, 2 ;
     stdem , 0.15 , 0.00005, 2 ;
     gamma , 0.6 , -0.9999, .9999 ;
     eps , 2 , 0.00005, 20;
end;

//stoch_simul (nograph,  nofunctions,   order=1,  ar=1, periods=2000)  loghn_1 loghn_2 loghn_3 loghn_4 ; 

 estimated_params_init(use_calibration);
 end;

 estimation( datafile=benhabibX, plot_priors=0, order=1, mode_compute=8, first_obs=2);


